
import HCard;
import java.lang.Integer;
import java.lang.Character;
import java.awt.*;

// immutable Bridge (or Solitare) style cards
//   support for Ace high/low

public class HStdCard extends HCard 
{
	
	// value = suit (16,32,48,64) + rank (0-14)
	protected int value;
	
	// suits (binary masks)
	static final int CLUB     = 0x10;
	static final int DIAMOND  = 0x20; 
	static final int HEART    = 0x40;
	static final int SPADE    = 0x80;
	static final int SUITONLY = 0xf0;

	// non-numeric ranks
	static final int JACK     = 11;
	static final int QUEEN    = 12;
	static final int KING     = 13;
	static final int ACE      = 14;
	static final int ACEHIGH  = 14;
	static final int ACELOW   = 1;
	static final int RANKONLY = 0x0f;
	
	// construction and creation validation
	protected HStdCard(int suit, int rank)
	{
		value = suit | rank;
	}

	public HStdCard create(int suit, int rank)
	{
		if (validSuitAndRank(suit, rank))
			return new HStdCard(suit,rank);
		else
			return null;
	}

	public boolean validSuitAndRank(int suit, int rank)
	{
		if ((suit != CLUB)    &&
			(suit != DIAMOND) &&
			(suit != HEART)   &&
			(suit != SPADE))
			return false;

		if ((rank < 1) || 
			(rank > 14))
			return false;

		return true;
	}

	public String toString()
	{
		int suit = value & SUITONLY;
		int rank = value & RANKONLY;
		char suitChar = 'E';
		char rankChar = 'E';

		if (suit == CLUB)     suitChar = 'C';
		if (suit == DIAMOND)  suitChar = 'D';
		if (suit == HEART)    suitChar = 'H';
		if (suit == SPADE)    suitChar = 'S';
		
		if ((rank > 1) && (rank < 10))
			rankChar = java.lang.Character.forDigit(rank, 10);

		if (rank == 10)       rankChar = 'T';
		if (rank == JACK)     rankChar = 'J';
		if (rank == QUEEN)    rankChar = 'Q';
		if (rank == KING)     rankChar = 'K';
		if (rank == ACE)      rankChar = 'A';
		if (rank == ACELOW)   rankChar = 'A';

		return (new Character(suitChar)).toString() + 
			   (new Character(rankChar)).toString();
	}

	// display stuff
	public static String imagePrefix =
		"cards/card";

	public static Image backImage = 
		Toolkit.getDefaultToolkit().getImage(imagePrefix + "Back.gif");
	
	public static Hashtable cardImages = createImages();

	public static Hashtable createImages()
	{
		Hashtable images = new Hashtable(52);
		
		for(Enumeration i = mydeck.elements();
			i.hasMoreElements();)
			{
				Object elem = i.nextElement();
				images.put(elem, 
					Toolkit.getDefaultToolkit().getImage("cards/card" +
					elem.toString() + ".gif"));
			}
	
		return images;
	}

}
